@@ -87,6 +87,20 @@ def status_corpora_markdown(maxcolwidth=45,link=False,**attrs):
 
 
 
+def yield_corpora_meta(corpora,incl_meta=[]):
+    o=[]
+    for cname in tqdm(corpora,desc='Loading metadata'):
+        C=load_corpus(cname) if type(cname)==str else cname
+        for dx in C.meta_iter():
+            if incl_meta:
+                dx=dict((k,dx.get(k,'')) for k in incl_meta)
+                #for key in set(dx.keys())-set(incl_meta): del dx[key]
+            yield dx
+
+def small_meta(corpora,incl_meta=['id','corpus','year','genre','major_genre','canon_genre']):
+    return pd.DataFrame(yield_corpora_meta(corpora,incl_meta))
+
+
 
 def install(cname_or_id_or_C,*x,**y):
     C=load_corpus(cname_or_id_or_C) if type(cname_or_id_or_C)==str else cname_or_id_or_C
@@ -777,18 +791,20 @@ def do_gen_mfw_grp(group,*x,**y):
         {'word':w, 'count':c}
         for w,c in countd.items()
         if is_valid_mfw_word(w)
-    ]).sort_values('count',ascending=False)
-    total=df['count'].sum()
-    # if y.get('by_fpm'):
-        # df['count']=df['count'] / 1000000
-    df['fpm']=df['count']/total*1000000
-    df['rank']=[i+1 for i in range(len(df))]
+    ])
+    if 'count' in df.columns:
+        df=df.sort_values('count',ascending=False)
+        total=df['count'].sum()
+        # if y.get('by_fpm'):
+            # df['count']=df['count'] / 1000000
+        df['fpm']=df['count']/total*1000000
+        df['rank']=[i+1 for i in range(len(df))]
     if 'index' in df.columns: df=df.drop('index',1)
     return df
 
 
 
-def load(name_or_id,load_meta=False,install_if_nec=True,**y):
+def load(name_or_id,load_meta=False,install_if_nec=False,**y):
     return load_corpus(name_or_id,load_meta=load_meta,install_if_nec=install_if_nec,**y)
 #################################################################
 
